@@ -359,6 +359,7 @@ def _get_conn(dsn):
              'postgresql' : _conn_pgsql,
              'mysql'      : _conn_mysql,
              'oracle'     : _conn_oracle,
+             'mssql'      : _conn_mssql,
             }
     return conns[protocol](location)
     
@@ -382,13 +383,13 @@ def _conn_pgsql(location):
         elif type_ in DATETIME: return 'Float64'
         elif type_ in NUMBER: return 'Float64'  # assume Float64 for numbers, user can override
 
-    # Build connection. psycopg uses a connection string.
-    user, password, host, port, dbname = split_dsn(location)
-    conn_str = ["dbname=%s" % dbname]
-    for name in ['host', 'port', 'user', 'password']:
-        var = locals()[name]
-        if var is not None:
-            conn_str.append("%s=%s" % (name, var))
+    user, passwd, host, port, dbname = split_dsn(location)
+    conn_str = [('dbname', dbname)]
+    if host: conn_str.append(('host', host))
+    if port: conn_str.append(('port', str(port)))
+    if user: conn_str.append(('user', user))
+    if passwd: conn_str.append(('password', passwd))
+    conn_str = ['='.join(t) for t in conn_str]
     conn_str = ' '.join(conn_str)
 
     conn = connect(conn_str)
@@ -405,13 +406,12 @@ def _conn_mysql(location):
         elif type_ in DATETIME: return 'Float64'
         elif type_ in NUMBER: return 'Float64'  # assume Float64 for numbers, user can override
 
-    # Build connection. MySQLdb uses kwargs for connection.
     user, passwd, host, port, db = split_dsn(location)
-
-    kwargs = {}
-    for name in ['user', 'passwd', 'host', 'port', 'db']:
-        var = locals()[name]
-        if var is not None: kwargs[name] = var
+    kwargs = {'db': db}
+    if host: kwargs['host'] = host
+    if port: kwargs['port'] = port
+    if user: kwargs['user'] = user
+    if passwd: kwargs['passwd'] = passwd
 
     conn = connect(**kwargs)
     return conn, type_convert
@@ -419,7 +419,7 @@ def _conn_mysql(location):
 
 def _conn_oracle(location):
     """Oracle connection."""
-    from cx_Oracle import connect, STRING, DATETIME, NUMBER
+    from cx_Oracle import connect, STRING, DATETIME, NUMBER, makedsn
     
     # Convert between cursor.description[1] types and DAP types.
     def type_convert(type_):
@@ -427,22 +427,39 @@ def _conn_oracle(location):
         elif type_ in DATETIME: return 'Float64'
         elif type_ in NUMBER: return 'Float64'  # assume Float64 for numbers, user can override
 
-    # Build connection. cx_Oracle uses kwargs for connection.
-    user, password, host, port, db = split_dsn(location)
-    dsn = [host]
-    if post: dsn.append(':%s' % port)
-    dsn.append('/%s' % db)
-    dsn = ''.join(dsn)
-
-    kwargs = {}
-    for name in ['user', 'password', 'dsn']:
-        var = locals()[name]
-        if var is not None: kwargs[name] = var
+    user, passwd, host, port, db = split_dsn(location)
+    kwargs = {'dsn': makedsn(host, port, db)}
+    if user: kwargs['user'] = user
+    if passwd: kwargs['password'] = passwd
 
     conn = connect(**kwargs)
     return conn, type_convert
 
 
+def _conn_mssql(location):
+    """MS SQL connection."""
+    from adodbapi import connect, STRING, DATETIME, NUMBER
+
+    # Convert between cursor.description[1] types and DAP types.
+    def type_convert(type_):
+        if type_ in STRING: return 'String'
+        elif type_ in DATETIME: return 'Float64'
+        elif type_ in NUMBER: return 'Float64'  # assume Float64 for numbers, user can override
+
+    user, passwd, host, port, db = split_dsn(location)
+    conn_str = [('Driver', '{SQL Server}'),
+                ('Database', db)]
+    if host: conn_str.append(('Server', host))
+    if port: conn_str.append(('Port', str(port)))
+    if user: conn_str.append(('Uid', user))
+    if passwd: conn_str.append(('Pwd', passwd))
+    conn_str = ['='.join(t) for t in conn_str]
+    conn_str = ';'.join(conn_str)
+
+    conn = connect(conn_str)
+    return conn, type_convert
+
+
 def _test():
     import doctest
     doctest.testmod()
